<?php

namespace Drupal\datex\Datex;

use Drupal\Core\Datetime\DrupalDateTime;

class DatexDrupalDateTime extends DrupalDateTime {

  public function __construct($time = 'now', $timezone = NULL, array $settings = []) {
    parent::__construct($time, $timezone, $settings);
  }

  function format($format, $settings = []) {
    $langcode = !empty($settings['langcode']) ? $settings['langcode'] : $this->langcode;
    $value = '';
    try {
      if (!$this->hasErrors()) {
        if (isset($settings['timezone'])) {
          $tz = new \DateTimeZone($settings['timezone']);
        }
        else {
          $tz = $this->getTimezone();
        }
        return datex_factory($tz, 'en')
          ->setTimestamp($this->getTimestamp())
          ->format($format);
      }
    }
    catch (\Exception $e) {
      $this->errors[] = $e->getMessage();
    }
    return $value;
  }

  static function convert(DrupalDateTime $dateTime) {
    $me = new DatexDrupalDateTime(
      $dateTime->getTimestamp(),
      $dateTime->getTimezone(),
      ['langcode' => $dateTime->langcode]
    );

    $me->formatTranslationCache = $dateTime->formatTranslationCache;
    $me->stringTranslation = $dateTime->stringTranslation;

    $me->inputTimeRaw = $dateTime->inputTimeRaw;
    $me->inputTimeAdjusted = $dateTime->inputTimeAdjusted;
    $me->inputTimeZoneRaw = $dateTime->inputTimeZoneRaw;
    $me->inputTimeZoneAdjusted = $dateTime->inputTimeZoneAdjusted;
    $me->inputFormatRaw = $dateTime->inputFormatRaw;
    $me->inputFormatAdjusted = $dateTime->inputFormatAdjusted;
    $me->langcode = $dateTime->langcode;
    $me->errors = $dateTime->errors;
    $me->dateTimeObject = $dateTime->dateTimeObject;

    $me->setTimestamp($dateTime->getTimestamp());

    return $me;
  }

}